/*  Crystality Plugin - A plugin for remastering mp3 sound in realtime
 *
 *  Copyright (C) 2001 Rafal Bosak <gyver@fanthom.irc.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>

#include <xmms/plugin.h>
#include <xmms/configfile.h>
#include "crystality.h"

int process_sound_xmms(gpointer *data, gint length, AFormat afmt, gint srate, gint nch);

EffectPlugin extender_ep =
{
	NULL, NULL,
	"Crystality Plugin v0.9",
	init,
	cleanup,
	about,
	configure,
	process_sound_xmms
};

EffectPlugin *get_eplugin_info(void)
{
	return &extender_ep;
}

int read_config(void)
{
    ConfigFile *cfg;
    gchar *filename;

    filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
    cfg = xmms_cfg_open_file(filename);

    if (cfg && xmms_cfg_read_int(cfg, "Extender-DSP", "expander", &bext_level)){
	xmms_cfg_read_int(cfg, "Extender-DSP", "echo", &echo_level);
	xmms_cfg_read_int(cfg, "Extender-DSP", "stereo", &stereo_level);
	xmms_cfg_read_int(cfg, "Extender-DSP", "filter", &filter_level);
	xmms_cfg_read_int(cfg, "Extender-DSP", "feedback", &feedback_level);
	xmms_cfg_read_int(cfg, "Extender-DSP", "harmonics", &harmonics_level);
	xmms_cfg_free(cfg);
       	g_free(filename);
	return 1;
    } else {
	set_defaults();
       	g_free(filename);
	return -1;
    }
}

int write_config(void)
{
    ConfigFile *cfg;
    gchar *filename;

    filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
    cfg = xmms_cfg_open_file(filename);
    if (!cfg) cfg = xmms_cfg_new();
    if (cfg) {
	xmms_cfg_write_int(cfg, "Extender-DSP", "expander", bext_level);
	xmms_cfg_write_int(cfg, "Extender-DSP", "echo", echo_level);
	xmms_cfg_write_int(cfg, "Extender-DSP", "stereo", stereo_level);
	xmms_cfg_write_int(cfg, "Extender-DSP", "filter", filter_level);
	xmms_cfg_write_int(cfg, "Extender-DSP", "feedback", feedback_level);
	xmms_cfg_write_int(cfg, "Extender-DSP", "harmonics", harmonics_level);
       	xmms_cfg_write_file(cfg, filename);
	xmms_cfg_free(cfg);
	return 1;
    } else {
       	g_free(filename);
	return -1;
    }
}

int process_sound_xmms(gpointer *data, gint length, AFormat afmt, gint srate, gint nch)
{
    process_sound((void*)*data, (int)length, (int)afmt, (int)srate, (int)nch);
    return length;
}

